module Context where
import Syntax 

--                     b    em   i    s    tt   u   size color
 
data Context = Context Bool Bool Bool Bool Bool Int Int Color deriving (Show, Eq)

data Color = Color Char deriving (Show, Eq)

enterTag :: Context -> Tag -> Context
-- When entering a B tag, the b attribute must be set to true
enterTag (Context b em i s tt u size color) B = Context True em i s tt u size color

-- When entering an EM tag, we must do nothing if the S flag is set
-- otherwise, the em flag must be inverted
enterTag con@(Context b em i True tt u size color) EM = con
enterTag (Context b em i False tt u size color) EM = Context b (not em) i False tt u size color

-- When entering an I tag, the i attribute must be set
enterTag (Context b em i s tt u size color) I = Context b em True s tt u size color

-- When entering a PL tag, the u,s,em,i,b and tt flags must all be set to False
enterTag (Context b em i s tt u size color) PL = Context False False False False False 0 size color

-- When entering a S tag, the s flag must be set, and the em flag must be unset
enterTag (Context b em i s tt u size color) S = Context b False i True tt u size color

-- when entering a TT tag, the tt attribute must be set to true
enterTag (Context b em i s tt u size color) TT = Context b em i s True u size color

-- when entering U tag, we must increase the u attribute
-- we shall increase it only to 3, and 3 shall be that maximum number assigned to u
-- 4 shall thou not assign to u, nor shall thou set it to -1
enterTag con@(Context b em i s tt u size color) U = if (u <= 2) then Context b em i s tt (u+1) size color
											    else con

-- When entering an SZ tag, we change the size
enterTag (Context b em i s tt u size color) (Sz x) = Context b em i s tt u x color

-- When entering a C tag, we set the color attribute accordingly
enterTag (Context b em i s tt u size color) (C x) = Context b em i s tt u size (Color x)





-- Check whether a given tag would change the given context
changes :: Tag -> Context -> Bool
changes tag context = not (context==(enterTag context tag))

-- let's see if the changes function does what is has to do

testChanges :: Bool

testChanges = if (changes B (Context True True True True True 0 0 (Color 'e'))==True) then False
			  else (if (changes B (Context False True True True True 0 0 (Color 'e'))==False) then False
			        else True
                   )



getB :: Context -> Bool
getB (Context b em i s tt u size color) = b

getEM :: Context -> Bool
getEM (Context b em i s tt u size color) = em

getI :: Context -> Bool
getI (Context b em i s tt u size color) = i

getS :: Context -> Bool
getS (Context b em i s tt u size color) = s

getTT :: Context -> Bool
getTT (Context b em i s tt u size color) = tt

getU :: Context -> Int
getU (Context b em i s tt u size color) = u

getSIZE :: Context -> Int
getSIZE (Context b em i s tt u size color) = size

getCOLOR :: Context -> Color
getCOLOR (Context b em i s tt u size color) = color

emptyContext :: Context
emptyContext = (Context False False False False False 0 (-1) (Color '*'))
