module Parser where

import Syntax

import Context

import IO

import Monad

-- only for hugs
-- isEOF = hugsIsEOF

parse :: IO Document

parse = liftM head $ ps [[]] [] [emptyContext]

ps :: [Document] -> [Tag] -> [Context] -> IO [Document]
ps docs tags ct = do 
                     eof <- isEOF
		     if eof then return docs
		            else do 
			            c         <- getChar
			   	    (d,t,ct') <- eatChar c docs tags ct
				    ps d t ct'

eatChar :: Char -> [Document] -> [Tag] -> [Context] -> IO ([Document], [Tag], [Context])

eatChar '<'  docs tags ct     = eatTag docs tags ct

eatChar ' ' docs@((Ws:r):t) tags ct@(Context _ _ _ _ False _ _ _:_)  = return (docs,tags,ct)
eatChar ' '  (h:t) tags ct@(Context _ _ _ _ False _ _ _:_)  = return ((Ws:h):t,tags,ct)
eatChar '\n' docs@((Ws:r):t) tags ct@(Context _ _ _ _ False _ _ _:_)  = return (docs,tags,ct)
eatChar '\n' (h:t) tags ct@(Context _ _ _ _ False _ _ _:_)  = return ((Ws:h):t,tags,ct)
eatChar '\t' docs@((Ws:r):t) tags ct@(Context _ _ _ _ False _ _ _:_)  = return (docs,tags,ct)
eatChar '\t' (h:t) tags ct@(Context _ _ _ _ False _ _ _:_)  = return ((Ws:h):t,tags,ct)
eatChar '\r' docs@((Ws:r):t) tags ct@(Context _ _ _ _ False _ _ _:_) = return (docs, tags,ct)
eatChar '\r' (h:t) tags ct@(Context _ _ _ _ False _ _ _:_)  = return ((Ws:h):t,tags,ct)

eatChar c ( ( (Txt l) : hr) : tr) tags ct = return ( ((Txt (c:l)) : hr) : tr  , tags, ct)
eatChar c (h:t) tags ct = return (((Txt [c]):h):t,tags,ct)

eatTag :: [Document] -> [Tag] -> [Context] -> IO ([Document],[Tag],[Context])
eatTag docs tags ct =
  do 
     c <- getChar
     case c of
       '/' -> restTag >> popTag docs tags ct
       'E' -> restTag >> pushEM docs tags ct
       'T' -> restTag >> pushTT docs tags ct
       'P' -> restTag >> pushPlain docs tags ct
       _   -> restTag >> pushTag c docs tags ct
       
popTag :: [Document] -> [Tag] -> [Context] -> IO ([Document],[Tag],[Context])
-- currently no correct TT restoration
popTag ([]:r) (t:ts) ct = return (r,ts,ct) -- leave out empty tags
popTag (x:(y:r)) (t:ts) (c':c:cs) = if c == c' then return ((x++y):r,ts,c:cs)
				               else return (((Tag t x):y):r,ts,c:cs)

pushEM :: [Document] -> [Tag] -> [Context] -> IO ([Document],[Tag],[Context])
pushEM docs tags (c:cs) = return ([]:docs,EM:tags,(enterTag c EM):c:cs)
 
pushTT :: [Document] -> [Tag] -> [Context] -> IO ([Document],[Tag],[Context])
pushTT docs tags (c:cs) = return ([]:docs,TT:tags,(enterTag c TT):c:cs)

pushPlain :: [Document] -> [Tag] -> [Context] -> IO ([Document],[Tag],[Context])
pushPlain docs tags (c:cs) = return ([]:docs,PL:tags,(enterTag c PL):c:cs)
 
pushTag :: Char -> [Document] -> [Tag] -> [Context] -> IO ([Document],[Tag],[Context])
pushTag x docs tags (c:cs) = let t = getTag x
			     in return ([]:docs,t:tags,(enterTag c t):c:cs)

restTag :: IO ()
restTag = 
  do 
     c <- getChar
     case c of 
       '>' -> return ()
       _   -> restTag
